<?php
use CRM_Bij1migratie_Mapping_AirtableMapping as AirtableMapping;
use CRM_Bij1migratie_Mapping_BetalingenMapping as BetalingenMapping;
use CRM_Bij1migratie_Mapping_ChargebacksMapping as ChargebacksMapping;
use CRM_Bij1migratie_Mapping_PersonenMapping as PersonenMapping;
use CRM_Bij1migratie_ExtensionUtil as E;

/**
 * Class BIJ1 migratie service.
 * Bevat logica over het migratie formulier, evenals
 * getters/setters en wat methods om waarden te mappen.
 *
 * @author Erik Hommel (CiviCooP) <erik.hommel@civicoop.org>
 * @date 9 Dec 021
 * @license AGPL-3.0
 */
class CRM_Bij1migratie_Bij1MigratieService {
  /**
   * @var CRM_Bij1migratie_Bij1MigratieService
   */
  protected static $singleton;
  protected $_mijnheerAanhef = NULL;
  protected $_mevrouwAanhef = NULL;
  protected $_manGeslacht = NULL;
  protected $_onbekendGeslacht = NULL;
  protected $_vrouwGeslacht = NULL;
  protected $_contributieFinancialTypeId = NULL;
  protected $_donatieFinancialTypeId = NULL;
  protected $_chargeBackContributionStatusId = NULL;
  protected $_completedContributionStatusId = NULL;
  protected $_pendingContributionStatusId = NULL;
  protected $_failedContributionStatusId = NULL;
  protected $_cancelledContributionStatusId = NULL;
  protected $_overdueContributionStatusId = NULL;
  protected $_migratieGroupId = NULL;

  /**
   * CRM_Bij1migratie_Bij1MigratieService constructor.
   */
  public function __construct() {
    if (!self::$singleton) {
      self::$singleton = $this;
    }
  }

  /**
   * @return \CRM_Bij1migratie_Bij1MigratieService
   */
  public static function getInstance() {
    if (!self::$singleton) {
      self::$singleton = new CRM_Bij1migratie_Bij1MigratieService();
    }
    return self::$singleton;
  }

  /**
   * @param int $id
   */
  public function setMigratieGroepId(int $id) {
    $this->_migratieGroupId = $id;
  }

  /**
   * @return null
   */
  public function getMigratieGroepId() {
    return $this->_migratieGroupId;
  }

  /**
   * @param int $value
   */
  public function setMijnheerAanhef(int $value) {
    $this->_mijnheerAanhef = $value;
  }

  /**
   * @return null
   */
  public function getMijnheerAanhef() {
    return $this->_mijnheerAanhef;
  }

  /**
   * @param int $value
   */
  public function setMevrouwAanhef(int $value) {
    $this->_mevrouwAanhef = $value;
  }

  /**
   * @return null
   */
  public function getMevrouwAanhef() {
    return $this->_mevrouwAanhef;
  }

  /**
   * @param int $value
   */
  public function setManGeslacht(int $value) {
    $this->_manGeslacht = $value;
  }

  /**
   * @return null
   */
  public function getManGeslacht() {
    return $this->_manGeslacht;
  }

  /**
   * @param int $value
   */
  public function setOnbekendGeslacht(int $value) {
    $this->_onbekendGeslacht = $value;
  }

  /**
   * @return null
   */
  public function getOnbekendGeslacht() {
    return $this->_onbekendGeslacht;
  }

  /**
   * @param int $value
   */
  public function setVrouwGeslacht(int $value) {
    $this->_vrouwGeslacht = $value;
  }

  /**
   * @return null
   */
  public function getVrouwGeslacht() {
    return $this->_vrouwGeslacht;
  }

  /**
   * @param int $id
   */
  public function setContributieFinancialTypeId(int $id) {
    $this->_contributieFinancialTypeId = $id;
  }

  /**
   * @return null
   */
  public function getContributieFinancialTypeId() {
    return $this->_contributieFinancialTypeId;
  }

  /**
   * @param int $id
   */
  public function setDonatieFinancialTypeId(int $id) {
    $this->_donatieFinancialTypeId = $id;
  }

  /**
   * @return null
   */
  public function getDonatieFinancialTypeId() {
    return $this->_donatieFinancialTypeId;
  }

  /**
   * @param int $id
   */
  public function setCancelledContributionStatusId(int $id) {
    $this->_cancelledContributionStatusId = $id;
  }

  /**
   * @return null
   */
  public function getCancelledContributionStatusId() {
    return $this->_cancelledContributionStatusId;
  }

  /**
   * @param int $id
   */
  public function setChargeBackContributionStatusId(int $id) {
    $this->_chargeBackContributionStatusId = $id;
  }

  /**
   * @return null
   */
  public function getChargeBackContributionStatusId() {
    return $this->_chargeBackContributionStatusId;
  }

  /**
   * @param int $id
   */
  public function setCompletedContributionStatusId(int $id) {
    $this->_completedContributionStatusId = $id;
  }

  /**
   * @return null
   */
  public function getCompletedContributionStatusId() {
    return $this->_completedContributionStatusId;
  }

  /**
   * @param int $id
   */
  public function setFailedContributionStatusId(int $id) {
    $this->_failedContributionStatusId = $id;
  }

  /**
   * @return null
   */
  public function getFailedContributionStatusId() {
    return $this->_failedContributionStatusId;
  }

  /**
   * @param int $id
   */
  public function setOverdueContributionStatusId(int $id) {
    $this->_overdueContributionStatusId = $id;
  }

  /**
   * @return null
   */
  public function getOverdueContributionStatusId() {
    return $this->_overdueContributionStatusId;
  }

  /**
   * @param int $id
   */
  public function setPendingContributionStatusId(int $id) {
    $this->_pendingContributionStatusId = $id;
  }

  /**
   * @return null
   */
  public function getPendingContributionStatusId() {
    return $this->_pendingContributionStatusId;
  }

  /**
   * Function om mogelijke separatoren voor csv bestand op te halen
   *
   * @return string[]
   */
  public function getSeparatorList() {
    return [",", ";", ":", "~"];
  }

  /**
   * Method om de separator voor het csv bestand te halen
   *
   * @param $separatorId
   * @return string
   */
  public function getSeparator($separatorId) {
    switch ($separatorId) {
      case 0:
        return ",";
      case 1:
        return ";";
      case 3:
        return ":";
      case 2:
        return "~";
      default:
        return "";
    }
  }

  /**
   * Method om formValues te schonen
   *
   * @param $sourceValues
   * @return array
   */
  public function cleanFormValues($sourceValues) {
    $result = [];
    foreach ($sourceValues as $sourceKey => $sourceValue) {
      if ($sourceKey != "qfKey" && $sourceKey != "entryURL" && substr($sourceKey,0,4) != "_qf_") {
        $result[$sourceKey] = $sourceValue;
      }
    }
    return $result;
  }

  /**
   * Method om de migratie kolom mappings op te halen
   *
   * @param string $migratieType
   * @return false|array
   * @throws
   */
  public function getMigratieColumns(string $migratieType) {
    switch ($migratieType) {
      case "A":
        return AirtableMapping::$MAPPING;
      case "B":
        return BetalingenMapping::$MAPPING;
      case "C":
        return ChargebacksMapping::$MAPPING;
      default:
        return PersonenMapping::$MAPPING;
    }
  }

  /**
   * Omzetten land naar country id voor airtable
   *
   * @param $airtableLand
   * @return int|void
   */
  public function haalAirtableLand($airtableLand) {
    $airtableLand = strtolower($airtableLand);
    switch ($airtableLand) {
      case "belgie":
      case "belgië":
        $isoCode = "BE";
        break;
      case "brazilië":
        $isoCode = "BR";
        break;
      case "canada":
        $isoCode = "CA";
        break;
      case "curacao":
      case "curaçao":
        $isoCode = "CW";
        break;
      case "denemarken":
        $isoCode = "DK";
        break;
      case "duitsland":
      case "deutschland":
        $isoCode = "DE";
        break;
      case "engeland":
      case "verenigd koninkrijk":
        $isoCode = "GB";
        break;
      case "finland":
        $isoCode = "FI";
        break;
      case "frankrijk":
        $isoCode = "FR";
        break;
      case "griekenland":
        $isoCode = "GR";
        break;
      case "ierland":
        $isoCode = "IE";
        break;
      case "italië":
      case "italie":
        $isoCode = "IT";
        break;
      case "kenia":
        $isoCode = "KE";
        break;
      case "kuwait":
        $isoCode = "KW";
        break;
      case "mexico":
        $isoCode = "MX";
        break;
      case "oostenrijk":
        $isoCode = "AT";
        break;
      case "polen":
        $isoCode = "PL";
        break;
      case "portugal":
        $isoCode = "PT";
        break;
      case "spanje":
        $isoCode = "ES";
        break;
      case "verenigde staten":
        $isoCode = "US";
        break;
      case "zweden":
        $isoCode = "SE";
        break;
      case "zwitserland":
        $isoCode = "CH";
        break;
      case "suriname":
        $isoCode = "SR";
        break;
      default:
        $isoCode = "NL";
        break;
    }
    $landId = CRM_Core_DAO::singleValueQuery("SELECT id FROM civicrm_country WHERE iso_code = %1", [1 => [$isoCode, "String"]]);
    if ($landId) {
      return (int) $landId;
    }
  }

  /**
   * Method om betalingsmethode te vertalen
   *
   * @param string $betalingsinstrument
   * @return null
   */
  public function haalBetalingsinstrument(string $betalingsinstrument) {
    switch ($betalingsinstrument) {
      case "first":
        return Civi::service('bij1algemeen')->getFrstPaymentId();
        break;
      case "oneoff":
        return Civi::service('bij1algemeen')->getOoffPaymentId();
        break;
      case "recurring":
        return Civi::service('bij1algemeen')->getRcurPaymentId();
        break;
      default:
        return Civi::service('bij1algemeen')->getEftPaymentId();
        break;
    }
  }

  /**
   * Method om financieel type vast te stellen
   *
   * @param array $rij
   * @return null
   */
  public function haalFinancieelType(array $rij) {
    if (isset($rij['omschrijving']) && !empty($rij['omschrijving'])) {
      // TODO: is dit wel case insensitive?
      if (strpos($rij['omschrijving'], "donatie") !== FALSE) {
        return $this->_donatieFinancialTypeId;
      }
      if (strpos($rij['omschrijving'], "donation") !== FALSE) {
        return $this->_donatieFinancialTypeId;
      }
    }
    return $this->_contributieFinancialTypeId;
  }

  /**
   * Method om bijdrage status te bepalen aan de hand van de status in Mollie
   *
   * @param string $mollieStatus
   * @return null
   */
  public function haalBijdrageStatusId(string $mollieStatus) {
    switch (strtolower($mollieStatus)) {
      case "expired":
        $civiStatus = $this->getOverdueContributionStatusId();
        break;
      case "failed":
        $civiStatus = $this->getFailedContributionStatusId();
        break;
      case "pending":
        $civiStatus = $this->getPendingContributionStatusId();
        break;
      default:
        $civiStatus = $this->getCompletedContributionStatusId();
        break;
    }
    return $civiStatus;
  }

}
