<?php
use CRM_Bij1migratie_ExtensionUtil as E;

/**
 * Class voor BIJ1 migratie chargebacks
 *
 * @author Erik Hommel (CiviCooP) <erik.hommel@civicoop.org>
 * @date 7 feb 2022
 * @license AGPL-3.0
 */
class CRM_Bij1migratie_ChargeBack {
  private $_chargeBackStatusId;

  /**
   * CRM_Bij1migratie_ChargeBack constructor
   */
  public function __construct() {
    $this->_chargeBackStatusId = Civi::service('bij1Migratie')->getChargeBackContributionStatusId();
  }

  /**
   * Haal contributionId met transaction Id
   *
   * @param string $transactionId
   * @return false|int
   */
  public function haalContributionId(string $transactionId) {
    if (!empty($transactionId)) {
      try {
        $contributions = \Civi\Api4\Contribution::get()
          ->addSelect('id')
          ->addWhere('trxn_id', '=', $transactionId)
          ->execute();
        $contribution = $contributions->first();
        if ($contribution['id']) {
          return (int) $contribution['id'];
        }
      }
      catch (API_Exception $ex) {
      }
    }
    return FALSE;
  }

  /**
   * Verwerk chargeback in contributies.
   *
   * @param int $contributionId
   * @param array $data
   * @return void
   * @throws API_Exception
   * @throws \Civi\API\Exception\UnauthorizedException
   */
  public function verwerk(int $contributionId, array $data) {
    $chargeBack = \Civi\Api4\Contribution::update()
      ->addValue('contribution_status_id',$this->_chargeBackStatusId)
      ->addValue('trxn_id', $data['transactie_id'] . "/" . $data['chargeback_id'])
      ->addWhere('id', '=', $contributionId);
    if (!empty($data['chargeback_datum'])) {
      $chargeBackDatum = new DateTime($data['chargeback_datum']);
    }
    else {
      $chargeBackDatum = new DateTime('now');
    }
    $reden = NULL;
    if (!empty($data['reden_code'])) {
      $reden = $data['reden_code'];
    }
    if (!empty($data['reden_omschrijving'])) {
      if ($reden) {
        $reden .= " - " . $data['reden_omschrijving'];
      }
      else {
        $reden = $data['reden_omschrijving'];
      }
    }
    if ($reden) {
      $chargeBack->addValue('cancel_reason', $reden);
    }
    $chargeBack->addValue('cancel_date', $chargeBackDatum->format("Y-m-d H:i:s"));
    $chargeBack->execute();
  }
}
