<?php

use CRM_Bij1migratie_Exception_MappingException as MappingException;

/**
 * Mapping om centraal bij te houden welke kolom op welke positie wordt verwacht in de gegeven $data.
 * mapping values indicate source column positions.
 * keys match references in code, not source column headers.
 *
 * @param array $data
 * @throws MappingException
 */
abstract class CRM_Bij1migratie_Mapping_AbstractMapping
{
    abstract protected function getColumns(): array;

    protected $data;

    public function __construct(array $data)
    {
        $this->data = $data;
    }

    public function get(string $name)
    {
        $columns = $this->getColumns();
        if (!isset($columns[$name])) {
            throw new MappingException(sprintf('Column %s not found in %s', $name, get_class($this)));
        }
        if (!isset($this->data[$columns[$name]])) {
            throw new MappingException(sprintf('Column %d with name %s not found in dataset', $columns[$name], $name));
        }

        return $this->data[$columns[$name]];
    }
}
