<?php
/**
 * @author Erik Hommel <erik.hommel@civicoop.org>
 * @date 13 Oct 2021
 * @license AGPL-3.0
 */
namespace Civi\Bij1migratie;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Definition;
use CRM_Bij1migratie_ExtensionUtil as E;

class Bij1MigratieContainer implements CompilerPassInterface {

  /**
   * You can modify the container here before it is dumped to PHP code.
   */
  public function process(ContainerBuilder $container) {
    $definition = new Definition('CRM_Bij1migratie_Bij1MigratieService');
    $definition->setFactory(['CRM_Bij1migratie_Bij1MigratieService', 'getInstance']);
    $this->setAanhef($definition);
    $this->setGeslacht($definition);
    $this->setFinancieelTypes($definition);
    $this->setContributionStatusIds($definition);
    $definition->setPublic(TRUE);
    $container->setDefinition('bij1Migratie', $definition);
  }

  /**
   * Zet property voor contributie status gereed
   *
   * @param $definition
   */
  private function setContributionStatusIds(&$definition) {
    $query = "SELECT cov.value, cov.name
        FROM civicrm_option_group AS cog JOIN civicrm_option_value AS cov ON cog.id = cov.option_group_id
        WHERE cog.name = %1";
    $dao = \CRM_Core_DAO::executeQuery($query, [1 => ["contribution_status", "String"]]);
    while ($dao->fetch()) {
      switch ($dao->name) {
        case "Cancelled":
          $definition->addMethodCall('setCancelledContributionStatusId', [(int) $dao->value]);
          break;
        case "Chargeback":
          $definition->addMethodCall('setChargeBackContributionStatusId', [(int) $dao->value]);
          break;
        case "Completed":
          $definition->addMethodCall('setCompletedContributionStatusId', [(int) $dao->value]);
          break;
        case "Failed":
          $definition->addMethodCall('setFailedContributionStatusId', [(int) $dao->value]);
          break;
        case "Overdue":
          $definition->addMethodCall('setOverdueContributionStatusId', [(int) $dao->value]);
          break;
        case "Pending";
            $definition->addMethodCall('setPendingContributionStatusId', [(int) $dao->value]);
          break;
      }
    }
  }

  /**
   * Zet properties voor financieel type donatie
   *
   * @param $definition
   */
  private function setFinancieelTypes(&$definition) {
    $query = "SELECT id, name FROM civicrm_financial_type WHERE name IN(%1, %2)";
    $dao = \CRM_Core_DAO::executeQuery($query, [
      1 => ["Contributie", "String"],
      2 => ["Donatie", "String"],
      ]);
    while ($dao->fetch()) {
      switch ($dao->name) {
        case "Contributie":
          $definition->addMethodCall('setContributieFinancialTypeId', [(int) $dao->id]);
          break;
        case "Donatie":
          $definition->addMethodCall('setDonatieFinancialTypeId', [(int) $dao->id]);
          break;
      }
    }
  }

  /**
   * Zet properties voor geslacht
   *
   * @param $definition
   */
  private function setGeslacht(&$definition) {
    $query = "SELECT cov.value, cov.name
        FROM civicrm_option_group AS cog JOIN civicrm_option_value AS cov ON cog.id = cov.option_group_id
        WHERE cog.name = %1;";
    $dao = \CRM_Core_DAO::executeQuery($query, [1 => ["gender", "String"]]);
    while ($dao->fetch()) {
      switch ($dao->name) {
        case "Female":
          $definition->addMethodCall('setVrouwGeslacht', [(int) $dao->value]);
          break;
        case "Male":
          $definition->addMethodCall('setManGeslacht', [(int) $dao->value]);
          break;
        case "Other":
          $definition->addMethodCall('setOnbekendGeslacht', [(int) $dao->value]);
          break;
      }
    }
  }

  /**
   * Zet properties voor aanhef
   *
   * @param $definition
   */
  private function setAanhef(&$definition) {
    $query = "SELECT cov.value, cov.name
        FROM civicrm_option_group AS cog JOIN civicrm_option_value AS cov ON cog.id = cov.option_group_id
        WHERE cog.name = %1 AND cov.name IN(%2, %3);";
    $dao = \CRM_Core_DAO::executeQuery($query, [
      1 => ["individual_prefix", "String"],
      2 => ["Mrs.", "String"],
      3 => ["Mr.", "String"],
    ]);
    while ($dao->fetch()) {
      switch ($dao->name) {
        case "Dhr":
        case "Mr.":
          $definition->addMethodCall('setMijnheerAanhef', [(int) $dao->value]);
          break;
        case "Mrs.":
          $definition->addMethodCall('setMevrouwAanhef', [(int) $dao->value]);
          break;
      }
    }
  }

}

