# bij1migratie

![Screenshot](/images/screenshot.png)

Helps migrate BIJ1's data sources (as CSV) to CiviCRM, including:

- Airtable
- Mollie
  - Customers
  - Payments
  - Chargebacks

The extension is licensed under [AGPL-3.0](LICENSE.txt).

## Requirements

* PHP v7.2+
* CiviCRM (tested: 5.48.0)
* [CiviCRM CLI](https://github.com/civicrm/cv)
* CiviCRM extensions:
  - [`action-provider`](https://civicrm.org/extensions/action-provider)
  - [`de.systopia.identitytracker`](https://github.com/systopia/de.systopia.identitytracker)
  - [`de.systopia.xcm`](https://github.com/systopia/de.systopia.xcm)
  - [`form-processor`](https://civicrm.org/extensions/form-processor)
  - [`org.project60.banking`](https://github.com/Project60/org.project60.banking/)
  - [`bij1algemeen`](https://code.bij1.org/bij1/crm-bij1algemeen)

### Testing

* [PHPUnit](https://phpunit.de/)
* [PHPStan](https://phpstan.org/)

### Development

* [CiviCRM Extension CLI](https://github.com/totten/civix/)

### Documentation

* [Python](https://python.org/)

## Installation (Web UI)

Learn more about installing CiviCRM extensions in the [CiviCRM Sysadmin Guide](https://docs.civicrm.org/sysadmin/en/latest/customize/extensions/).

## Installation (CLI, Zip)

Sysadmins and developers may download the `.zip` file for this extension and
install it with the command-line tool [cv](https://github.com/civicrm/cv).

```bash
cd <extension-dir>
cv dl bij1migratie@https://github.com/FIXME/bij1migratie/archive/master.zip
```

## Installation (CLI, Git)

Sysadmins and developers may clone the [Git](https://en.wikipedia.org/wiki/Git) repo for this extension and
install it with the command-line tool [cv](https://github.com/civicrm/cv).

```bash
git clone https://github.com/FIXME/bij1migratie.git
cv en bij1migratie
```

## Getting Started

- Zorg dat de volgende Civi instellingen ingesteld zijn:
  - `bij1_basisbedrag`
- In CiviCRM, onder tab `Beheer`, ga naar `Automation` en dan `Form processors`.
  Gebruik daar de import knop om de `.json` files uit map `form-processors` te importeren.
- Onder tab `Beheer`, ga naar `Migratie Bij1 -> CiviCRM`,
  e.g. http://localhost/wp-admin/admin.php?page=CiviCRM&q=civicrm%2Fbij1migratie%2Fform%2Fcsvselect.
- Vul het formulier in en start de import ([sample data](https://code.bij1.org/bij1/crm-bij1migratie/src/branch/main/data)), in volgorde:
  - Mollie klanten
  - Airtable
  - Betalingen
  - ChargeBacks

## Overview

- `data/**/*.csv`: anonymized source data samples
- `Mapping/*Mapping.php`: match names used in form processors to column positions in source CSVs.
- `CRM/Bij1migratie/CsvFile.php#sanitize*()`: column selection, transformations
- `CRM/Bij1migratie/Migratie.php#migreer*()`: checks, import by API, row-level error logging
- `form-processors/bij1_*_migratie.json`: how to process imported fields

## Testing

### Static analysis

```sh
phpstan analyse .
```

### Unit tests

- place this repository in a CiviCRM installation
- run:
```sh
phpunit tests
```

### Reloading extension code to container

Given an environment variable `$CIVICRM_BIJ1_PATH` pointing to your local
checkout of [`civicrm-bij1`](https://code.bij1.org/bij1/civicrm-bij1/):

```sh
export FLDR=/var/lib/docker/volumes/civicrm-bij1_civi-data/_data/wp-content/uploads/civicrm/ext/crm-bij1migratie/
mkdir -p $FLDR
sudo cp -r ./* $FLDR
docker compose -f $CIVICRM_BIJ1_PATH/docker-compose.yml exec civi bash -c "cv ext:enable bij1migratie"
```

## Build (this) documentation

Run:
```sh
pip install mkdocs
mkdocs serve
```

## Known Issues

(* FIXME *)
